@@ -54,7 +54,7 @@ static void set_boundaries(Agraph_t * g)
     Agnode_t *v;
     Agsym_t* pos_attr = GN_pos(g);
     glCompPoint pos;
-    float left = FLT_MAX, right = -FLT_MAX, top = FLT_MAX, bottom = -FLT_MAX;
+    float left = FLT_MAX, right = -FLT_MAX, top = -FLT_MAX, bottom = FLT_MAX;
 
     for (v = agfstnode(g); v; v = agnxtnode(g, v)) 
     {
@@ -280,9 +280,9 @@ static void renderSelectedNodes(Agraph_t * g)
 	nodeSize = ND_size(v);
 
 	if (defaultNodeShape == 0) 
-	    glVertex3f(pos.x,pos.y,pos.z+0.001);
+	    glVertex3f(pos.x, pos.y, pos.z + 0.001f);
 	else if (defaultNodeShape == 1) 
-	    drawCircle(pos.x,pos.y,nodeSize,pos.z+0.001);
+	    drawCircle(pos.x, pos.y, nodeSize, pos.z + 0.001f);
     }
     if(defaultNodeShape==0)
 	glEnd();
