@@ -1,6 +1,3 @@
-/* $Id$ $Revision$ */
-/* vim:set shiftwidth=4 ts=8: */
-
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
@@ -8,7 +5,7 @@
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
  *
- * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
 /* adjust.c
@@ -16,26 +13,33 @@
  * order to reduce/remove node overlaps.
  */
 
-#include "neato.h"
-#include "agxbuf.h"
-#include "utils.h"
-#include "ctype.h"
-#include "voronoi.h"
-#include "info.h"
-#include "edges.h"
-#include "site.h"
-#include "heap.h"
-#include "hedges.h"
-#include "digcola.h"
+#include <assert.h>
+#include <cgraph/alloc.h>
+#include <neatogen/neato.h>
+#include <cgraph/agxbuf.h>
+#include <common/utils.h>
+#include <ctype.h>
+#include <float.h>
+#include <math.h>
+#include <neatogen/voronoi.h>
+#include <neatogen/info.h>
+#include <neatogen/edges.h>
+#include <neatogen/site.h>
+#include <neatogen/heap.h>
+#include <neatogen/hedges.h>
+#include <neatogen/digcola.h>
 #if ((defined(HAVE_GTS) || defined(HAVE_TRIANGLE)) && defined(SFDP))
-#include "overlap.h"
+#include <neatogen/overlap.h>
 #endif
+#include <stdbool.h>
 #ifdef IPSEPCOLA
-#include "csolve_VPSC.h"
-#include "quad_prog_vpsc.h"
+#include <vpsc/csolve_VPSC.h>
+#include <neatogen/quad_prog_vpsc.h>
 #endif
+#include <cgraph/strcasecmp.h>
+#include <stddef.h>
 
-#define SEPFACT         0.8  /* default esep/sep */
+#define SEPFACT         0.8f  /* default esep/sep */
 
 static double margin = 0.05;	/* Create initial bounding box by adding
 				 * margin * dimension around box enclosing
@@ -47,7 +51,7 @@ static double incr = 0.05;	/* Increase bounding box by adding
 static int iterations = -1;	/* Number of iterations */
 static int useIter = 0;		/* Use specified number of iterations */
 
-static int doAll = 0;		/* Move all nodes, regardless of overlap */
+static bool doAll = false; // Move all nodes, regardless of overlap
 static Site **sites;		/* Array of pointers to sites; used in qsort */
 static Site **endSite;		/* Sentinel on sites array */
 static Point nw, ne, sw, se;	/* Corners of clipping window */
@@ -71,12 +75,8 @@ static void setBoundBox(Point * ll, Point * ur)
   */
 static void freeNodes(void)
 {
-    int i;
-    Info_t *ip = nodeInfo;
-
-    for (i = 0; i < nsites; i++) {
-	breakPoly(&ip->poly);
-	ip++;
+    for (size_t i = 0; i < nsites; i++) {
+	breakPoly(&nodeInfo[i].poly);
     }
     polyFree();
     infoinit();			/* Free vertices */
@@ -93,54 +93,34 @@ static void freeNodes(void)
  */
 static void chkBoundBox(Agraph_t * graph)
 {
-    char *marg;
     Point ll, ur;
-    int i;
-    double x, y;
-    double xmin, xmax, ymin, ymax;
-    double xmn, xmx, ymn, ymx;
-    double ydelta, xdelta;
-    Info_t *ip;
-    Poly *pp;
-    /* int          cnt; */
 
-    ip = nodeInfo;
-    pp = &ip->poly;
-    x = ip->site.coord.x;
-    y = ip->site.coord.y;
-    xmin = pp->origin.x + x;
-    ymin = pp->origin.y + y;
-    xmax = pp->corner.x + x;
-    ymax = pp->corner.y + y;
-    for (i = 1; i < nsites; i++) {
-	ip++;
-	pp = &ip->poly;
-	x = ip->site.coord.x;
-	y = ip->site.coord.y;
-	xmn = pp->origin.x + x;
-	ymn = pp->origin.y + y;
-	xmx = pp->corner.x + x;
-	ymx = pp->corner.y + y;
-	if (xmn < xmin)
-	    xmin = xmn;
-	if (ymn < ymin)
-	    ymin = ymn;
-	if (xmx > xmax)
-	    xmax = xmx;
-	if (ymx > ymax)
-	    ymax = ymx;
+    double x_min = DBL_MAX;
+    double y_min = DBL_MAX;
+    double x_max = -DBL_MAX;
+    double y_max = -DBL_MAX;
+    assert(nsites > 0);
+    for (size_t i = 0; i < nsites; ++i) {
+	Info_t *ip = &nodeInfo[i];
+	Poly *pp = &ip->poly;
+	double x = ip->site.coord.x;
+	double y = ip->site.coord.y;
+	x_min = fmin(x_min, pp->origin.x + x);
+	y_min = fmin(y_min, pp->origin.y + y);
+	x_max = fmax(x_max, pp->corner.x + x);
+	y_max = fmax(y_max, pp->corner.y + y);
     }
 
-    marg = agget(graph, "voro_margin");
-    if (marg && (*marg != '\0')) {
+    char *marg = agget(graph, "voro_margin");
+    if (marg && *marg != '\0') {
 	margin = atof(marg);
     }
-    ydelta = margin * (ymax - ymin);
-    xdelta = margin * (xmax - xmin);
-    ll.x = xmin - xdelta;
-    ll.y = ymin - ydelta;
-    ur.x = xmax + xdelta;
-    ur.y = ymax + ydelta;
+    double ydelta = margin * (y_max - y_min);
+    double xdelta = margin * (x_max - x_min);
+    ll.x = x_min - xdelta;
+    ll.y = y_min - ydelta;
+    ur.x = x_max + xdelta;
+    ur.y = y_max + ydelta;
 
     setBoundBox(&ll, &ur);
 }
@@ -151,18 +131,16 @@ static void chkBoundBox(Agraph_t * graph)
 static int makeInfo(Agraph_t * graph)
 {
     Agnode_t *node;
-    int i;
-    Info_t *ip;
     expand_t pmargin;
     int (*polyf)(Poly *, Agnode_t *, float, float);
 
-    nsites = agnnodes(graph);
+    assert(agnnodes(graph) >= 0);
+    nsites = (size_t)agnnodes(graph);
     geominit();
 
-    nodeInfo = N_GNEW(nsites, Info_t);
+    nodeInfo = gv_calloc(nsites, sizeof(Info_t));
 
     node = agfstnode(graph);
-    ip = nodeInfo;
 
     pmargin = sepFactor (graph);
 
@@ -174,7 +152,8 @@ static int makeInfo(Agraph_t * graph)
     }
 	
     else polyf = makePoly;
-    for (i = 0; i < nsites; i++) {
+    for (size_t i = 0; i < nsites; i++) {
+	Info_t *ip = &nodeInfo[i];
 	ip->site.coord.x = ND_pos(node)[0];
 	ip->site.coord.y = ND_pos(node)[1];
 
@@ -189,7 +168,6 @@ static int makeInfo(Agraph_t * graph)
 	ip->node = node;
 	ip->verts = NULL;
 	node = agnxtnode(graph, node);
-	ip++;
     }
     return 0;
 }
@@ -197,19 +175,19 @@ static int makeInfo(Agraph_t * graph)
 /* sort sites on y, then x, coord */
 static int scomp(const void *S1, const void *S2)
 {
-    Site *s1, *s2;
+    const Site *s1, *s2;
 
-    s1 = *(Site **) S1;
-    s2 = *(Site **) S2;
+    s1 = *(Site *const *) S1;
+    s2 = *(Site *const *) S2;
     if (s1->coord.y < s2->coord.y)
-	return (-1);
+	return -1;
     if (s1->coord.y > s2->coord.y)
-	return (1);
+	return 1;
     if (s1->coord.x < s2->coord.x)
-	return (-1);
+	return -1;
     if (s1->coord.x > s2->coord.x)
-	return (1);
-    return (0);
+	return 1;
+    return 0;
 }
 
  /* sortSites:
@@ -217,23 +195,17 @@ static int scomp(const void *S1, const void *S2)
   */
 static void sortSites(void)
 {
-    int i;
-    Site **sp;
-    Info_t *ip;
-
     if (sites == 0) {
-	sites = N_GNEW(nsites, Site *);
+	sites = gv_calloc(nsites, sizeof(Site*));
 	endSite = sites + nsites;
     }
 
-    sp = sites;
-    ip = nodeInfo;
     infoinit();
-    for (i = 0; i < nsites; i++) {
-	*sp++ = &(ip->site);
+    for (size_t i = 0; i < nsites; i++) {
+	Info_t *ip = &nodeInfo[i];
+	sites[i] = &ip->site;
 	ip->verts = NULL;
 	ip->site.refcnt = 1;
-	ip++;
     }
 
     qsort(sites, nsites, sizeof(Site *), scomp);
@@ -245,19 +217,16 @@ static void sortSites(void)
 
 static void geomUpdate(int doSort)
 {
-    int i;
-
     if (doSort)
 	sortSites();
 
     /* compute ranges */
-    xmin = sites[0]->coord.x;
-    xmax = sites[0]->coord.x;
-    for (i = 1; i < nsites; i++) {
-	if (sites[i]->coord.x < xmin)
-	    xmin = sites[i]->coord.x;
-	if (sites[i]->coord.x > xmax)
-	    xmax = sites[i]->coord.x;
+    xmin = DBL_MAX;
+    xmax = -DBL_MAX;
+    assert(nsites > 0);
+    for (size_t i = 0; i < nsites; ++i) {
+	xmin = fmin(xmin, sites[i]->coord.x);
+	xmax = fmax(xmax, sites[i]->coord.x);
     }
     ymin = sites[0]->coord.y;
     ymax = sites[nsites - 1]->coord.y;
@@ -268,13 +237,10 @@ static void geomUpdate(int doSort)
 
 static Site *nextOne(void)
 {
-    Site *s;
-
     if (nextSite < endSite) {
-	s = *nextSite++;
-	return (s);
+	return *nextSite++;
     } else
-	return ((Site *) NULL);
+	return NULL;
 }
 
 /* rmEquality:
@@ -294,9 +260,9 @@ static void rmEquality(void)
 
     while (ip < endSite) {
 	jp = ip + 1;
-	if ((jp >= endSite) ||
-	    ((*jp)->coord.x != (*ip)->coord.x) ||
-	    ((*jp)->coord.y != (*ip)->coord.y)) {
+	if (jp >= endSite ||
+	    (*jp)->coord.x != (*ip)->coord.x ||
+	    (*jp)->coord.y != (*ip)->coord.y) {
 	    ip = jp;
 	    continue;
 	}
@@ -304,16 +270,16 @@ static void rmEquality(void)
 	/* Find first node kp with position different from ip */
 	cnt = 2;
 	kp = jp + 1;
-	while ((kp < endSite) &&
-	       ((*kp)->coord.x == (*ip)->coord.x) &&
-	       ((*kp)->coord.y == (*ip)->coord.y)) {
+	while (kp < endSite &&
+	       (*kp)->coord.x == (*ip)->coord.x &&
+	       (*kp)->coord.y == (*ip)->coord.y) {
 	    cnt++;
 	    jp = kp;
 	    kp = jp + 1;
 	}
 
 	/* If next node exists and is on the same line */
-	if ((kp < endSite) && ((*kp)->coord.y == (*ip)->coord.y)) {
+	if (kp < endSite && (*kp)->coord.y == (*ip)->coord.y) {
 	    xdel = ((*kp)->coord.x - (*ip)->coord.x) / cnt;
 	    i = 1;
 	    for (jp = ip + 1; jp < kp; jp++) {
@@ -340,25 +306,20 @@ static void rmEquality(void)
 static int countOverlap(int iter)
 {
     int count = 0;
-    int i, j;
-    Info_t *ip = nodeInfo;
-    Info_t *jp;
 
-    for (i = 0; i < nsites; i++)
+    for (size_t i = 0; i < nsites; i++)
 	nodeInfo[i].overlaps = 0;
 
-    for (i = 0; i < nsites - 1; i++) {
-	jp = ip + 1;
-	for (j = i + 1; j < nsites; j++) {
-	    if (polyOverlap
-		(ip->site.coord, &ip->poly, jp->site.coord, &jp->poly)) {
+    for (size_t i = 0; i < nsites - 1; i++) {
+	Info_t *ip = &nodeInfo[i];
+	for (size_t j = i + 1; j < nsites; j++) {
+	    Info_t *jp = &nodeInfo[j];
+	    if (polyOverlap(ip->site.coord, &ip->poly, jp->site.coord, &jp->poly)) {
 		count++;
 		ip->overlaps = 1;
 		jp->overlaps = 1;
 	    }
-	    jp++;
 	}
-	ip++;
     }
 
     if (Verbose > 1)
@@ -392,13 +353,7 @@ static void increaseBoundBox(void)
   */
 static double areaOf(Point a, Point b, Point c)
 {
-    double area;
-
-    area =
-	(double) (fabs
-		  (a.x * (b.y - c.y) + b.x * (c.y - a.y) +
-		   c.x * (a.y - b.y)) / 2);
-    return area;
+    return fabs(a.x * (b.y - c.y) + b.x * (c.y - a.y) + c.x * (a.y - b.y)) / 2;
 }
 
  /* centroidOf:
@@ -433,9 +388,9 @@ static void newpos(Info_t * ip)
     while (q != NULL) {
 	area = areaOf(anchor->p, p->p, q->p);
 	centroidOf(anchor->p, p->p, q->p, &x, &y);
-	cx = cx + area * x;
-	cy = cy + area * y;
-	totalArea = totalArea + area;
+	cx += area * x;
+	cy += area * y;
+	totalArea += area;
 	p = q;
 	q = q->next;
     }
@@ -460,10 +415,9 @@ static void addCorners(void)
     double sed = dist_2(&ip->site.coord, &se);
     double ned = dist_2(&ip->site.coord, &ne);
     double d;
-    int i;
 
-    ip++;
-    for (i = 1; i < nsites; i++) {
+    for (size_t i = 1; i < nsites; i++) {
+	ip = &nodeInfo[i];
 	d = dist_2(&ip->site.coord, &sw);
 	if (d < swd) {
 	    swd = d;
@@ -484,7 +438,6 @@ static void addCorners(void)
 	    ned = d;
 	    nes = ip;
 	}
-	ip++;
     }
 
     addVertex(&sws->site, sw.x, sw.y);
@@ -503,14 +456,11 @@ static void addCorners(void)
   */
 static void newPos(void)
 {
-    int i;
-    Info_t *ip = nodeInfo;
-
     addCorners();
-    for (i = 0; i < nsites; i++) {
+    for (size_t i = 0; i < nsites; i++) {
+	Info_t *ip = &nodeInfo[i];
 	if (doAll || ip->overlaps)
 	    newpos(ip);
-	ip++;
     }
 }
 
@@ -537,20 +487,20 @@ static int vAdjust(void)
     int increaseCnt = 0;
     int cnt;
 
-    if (!useIter || (iterations > 0))
+    if (!useIter || iterations > 0)
 	overlapCnt = countOverlap(iterCnt);
 
-    if ((overlapCnt == 0) || (iterations == 0))
+    if (overlapCnt == 0 || iterations == 0)
 	return 0;
 
     rmEquality();
     geomUpdate(0);
-    voronoi(0, nextOne);
+    voronoi(nextOne);
     while (1) {
 	newPos();
 	iterCnt++;
 
-	if (useIter && (iterCnt == iterations))
+	if (useIter && iterCnt == iterations)
 	    break;
 	cnt = countOverlap(iterCnt);
 	if (cnt == 0)
@@ -563,22 +513,17 @@ static int vAdjust(void)
 
 	switch (badLevel) {
 	case 0:
-	    doAll = 1;
+	    doAll = true;
 	    break;
-/*
-      case 1:
-        doAll = 1;
-        break;
-*/
 	default:
-	    doAll = 1;
+	    doAll = true;
 	    increaseCnt++;
 	    increaseBoundBox();
 	    break;
 	}
 
 	geomUpdate(1);
-	voronoi(0, nextOne);
+	voronoi(nextOne);
     }
 
     if (Verbose) {
@@ -590,18 +535,14 @@ static int vAdjust(void)
     return 1;
 }
 
-static double rePos(Point c)
+static double rePos(void)
 {
-    int i;
-    Info_t *ip = nodeInfo;
     double f = 1.0 + incr;
 
-    for (i = 0; i < nsites; i++) {
-	/* ip->site.coord.x = f*(ip->site.coord.x - c.x) + c.x; */
-	/* ip->site.coord.y = f*(ip->site.coord.y - c.y) + c.y; */
-	ip->site.coord.x = f * ip->site.coord.x;
-	ip->site.coord.y = f * ip->site.coord.y;
-	ip++;
+    for (size_t i = 0; i < nsites; i++) {
+	Info_t *ip = &nodeInfo[i];
+	ip->site.coord.x *= f;
+	ip->site.coord.y *= f;
     }
     return f;
 }
@@ -611,23 +552,19 @@ static int sAdjust(void)
     int iterCnt = 0;
     int overlapCnt = 0;
     int cnt;
-    Point center;
-    /* double sc; */
 
-    if (!useIter || (iterations > 0))
+    if (!useIter || iterations > 0)
 	overlapCnt = countOverlap(iterCnt);
 
-    if ((overlapCnt == 0) || (iterations == 0))
+    if (overlapCnt == 0 || iterations == 0)
 	return 0;
 
     rmEquality();
-    center.x = (pxmin + pxmax) / 2.0;
-    center.y = (pymin + pymax) / 2.0;
     while (1) {
-	/* sc = */ rePos(center);
+	rePos();
 	iterCnt++;
 
-	if (useIter && (iterCnt == iterations))
+	if (useIter && iterCnt == iterations)
 	    break;
 	cnt = countOverlap(iterCnt);
 	if (cnt == 0)
@@ -644,18 +581,12 @@ static int sAdjust(void)
  /* updateGraph:
   * Enter new node positions into the graph
   */
-static void updateGraph(Agraph_t * graph)
+static void updateGraph(void)
 {
-    /* Agnode_t*    node; */
-    int i;
-    Info_t *ip;
-    /* char         pos[100]; */
-
-    ip = nodeInfo;
-    for (i = 0; i < nsites; i++) {
+    for (size_t i = 0; i < nsites; i++) {
+	Info_t *ip = &nodeInfo[i];
 	ND_pos(ip->node)[0] = ip->site.coord.x;
 	ND_pos(ip->node)[1] = ip->site.coord.y;
-	ip++;
     }
 }
 
@@ -670,9 +601,8 @@ static void updateGraph(Agraph_t * graph)
 double *getSizes(Agraph_t * g, pointf pad, int* n_elabels, int** elabels)
 {
     Agnode_t *n;
-    real *sizes = N_GNEW(Ndim * agnnodes(g), real);
+    double *sizes = gv_calloc(Ndim * agnnodes(g), sizeof(double));
     int i, nedge_nodes = 0;
-    int* elabs;
 
     for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
 	if (elabels && IS_LNODE(n)) nedge_nodes++;
@@ -683,7 +613,7 @@ double *getSizes(Agraph_t * g, pointf pad, int* n_elabels, int** elabels)
     }
 
     if (elabels && nedge_nodes) {
-	elabs = N_GNEW(nedge_nodes, int);
+	int* elabs = gv_calloc(nedge_nodes, sizeof(int));
 	nedge_nodes = 0;
 	for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
 	    if (IS_LNODE(n))
@@ -700,7 +630,7 @@ double *getSizes(Agraph_t * g, pointf pad, int* n_elabels, int** elabels)
  * Assumes g is connected and simple, i.e., we can have a->b and b->a
  * but not a->b and a->b
  */
-SparseMatrix makeMatrix(Agraph_t* g, int dim, SparseMatrix *D)
+SparseMatrix makeMatrix(Agraph_t* g, SparseMatrix *D)
 {
     SparseMatrix A = 0;
     Agnode_t *n;
@@ -709,13 +639,10 @@ SparseMatrix makeMatrix(Agraph_t* g, int dim, SparseMatrix *D)
     int nnodes;
     int nedges;
     int i, row;
-    int *I;
-    int *J;
-    real *val;
-    real v;
+    double v;
     int type = MATRIX_TYPE_REAL;
     Agsym_t* symD = NULL;
-    real* valD = NULL;
+    double* valD = NULL;
 
     if (!g)
 	return NULL;
@@ -727,14 +654,14 @@ SparseMatrix makeMatrix(Agraph_t* g, int dim, SparseMatrix *D)
     for (n = agfstnode(g); n; n = agnxtnode(g, n))
 	ND_id(n) = i++;
 
-    I = N_GNEW(nedges, int);
-    J = N_GNEW(nedges, int);
-    val = N_GNEW(nedges, real);
+    int *I = gv_calloc(nedges, sizeof(int));
+    int *J = gv_calloc(nedges, sizeof(int));
+    double *val = gv_calloc(nedges, sizeof(double));
 
     sym = agfindedgeattr(g, "weight");
     if (D) {
 	symD = agfindedgeattr(g, "len");
-	valD = N_NEW(nedges, real);
+	valD = gv_calloc(nedges, sizeof(double));
     }
 
     i = 0;
@@ -743,7 +670,7 @@ SparseMatrix makeMatrix(Agraph_t* g, int dim, SparseMatrix *D)
 	for (e = agfstout(g, n); e; e = agnxtout(g, e)) {
 	    I[i] = row;
 	    J[i] = ND_id(aghead(e));
-	    if (!sym || (sscanf(agxget(e, sym), "%lf", &v) != 1))
+	    if (!sym || sscanf(agxget(e, sym), "%lf", &v) != 1)
 		v = 1;
 	    val[i] = v;
 	/* edge length */
@@ -756,14 +683,14 @@ SparseMatrix makeMatrix(Agraph_t* g, int dim, SparseMatrix *D)
     }
 
     A = SparseMatrix_from_coordinate_arrays(nedges, nnodes, nnodes, I, J,
-					    val, type, sizeof(real));
+					    val, type, sizeof(double));
 
-    if (D) *D = SparseMatrix_from_coordinate_arrays(nedges, nnodes, nnodes, I, J, valD, type, sizeof(real));
+    if (D) *D = SparseMatrix_from_coordinate_arrays(nedges, nnodes, nnodes, I, J, valD, type, sizeof(double));
 
     free(I);
     free(J);
     free(val);
-    if (valD) free (valD);
+    free (valD);
 
     return A;
 }
@@ -772,12 +699,12 @@ SparseMatrix makeMatrix(Agraph_t* g, int dim, SparseMatrix *D)
 static int
 fdpAdjust (graph_t* g, adjust_data* am)
 {
-    SparseMatrix A0 = makeMatrix(g, Ndim, NULL);
+    SparseMatrix A0 = makeMatrix(g, NULL);
     SparseMatrix A = A0;
-    real *sizes;
-    real *pos = N_NEW(Ndim * agnnodes(g), real);
+    double *sizes;
+    double *pos = gv_calloc(Ndim * agnnodes(g), sizeof(double));
     Agnode_t *n;
-    int flag, i;
+    int flag = 0, i;
     expand_t sep = sepFactor(g);
     pointf pad;
 
@@ -791,24 +718,24 @@ fdpAdjust (graph_t* g, adjust_data* am)
     sizes = getSizes(g, pad, NULL, NULL);
 
     for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
-	real* npos = pos + (Ndim * ND_id(n));
+	double* npos = pos + Ndim * ND_id(n);
 	for (i = 0; i < Ndim; i++) {
 	    npos[i] = ND_pos(n)[i];
 	}
     }
 
-    if (!SparseMatrix_is_symmetric(A, FALSE)
-	|| A->type != MATRIX_TYPE_REAL) {
+    if (!SparseMatrix_is_symmetric(A, false) || A->type != MATRIX_TYPE_REAL) {
 	A = SparseMatrix_get_real_adjacency_matrix_symmetrized(A);
     } else {
 	A = SparseMatrix_remove_diagonal(A);
     }
 
     remove_overlap(Ndim, A, pos, sizes, am->value, am->scaling, 
-		   ELSCHEME_NONE, 0, NULL, NULL, mapBool (agget(g, "overlap_shrink"), TRUE), &flag);
+                   ELSCHEME_NONE, 0, NULL, NULL,
+                   mapBool(agget(g, "overlap_shrink"), true) ? TRUE : FALSE);
 
     for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
-	real *npos = pos + (Ndim * ND_id(n));
+	double *npos = pos + Ndim * ND_id(n);
 	for (i = 0; i < Ndim; i++) {
 	    ND_pos(n)[i] = npos[i];
 	}
@@ -828,15 +755,15 @@ fdpAdjust (graph_t* g, adjust_data* am)
 static int
 vpscAdjust(graph_t* G)
 {
-    int dim = 2;
+    enum { dim = 2 };
     int nnodes = agnnodes(G);
     ipsep_options opt;
-    pointf* nsize = N_GNEW(nnodes, pointf);
-    float** coords = N_GNEW(dim, float*);
-    float* f_storage = N_GNEW(dim * nnodes, float);
+    pointf *nsize = gv_calloc(nnodes, sizeof(pointf));
+    float* coords[dim];
+    float *f_storage = gv_calloc(dim * nnodes, sizeof(float));
     int i, j;
     Agnode_t* v;
-    expand_t margin;
+    expand_t exp_margin;
 
     for (i = 0; i < dim; i++) {
 	coords[i] = f_storage + i * nnodes;
@@ -845,7 +772,7 @@ vpscAdjust(graph_t* G)
     j = 0;
     for (v = agfstnode(G); v; v = agnxtnode(G, v)) {
 	for (i = 0; i < dim; i++) {
-	    coords[i][j] =  (float) (ND_pos(v)[i]);
+	    coords[i][j] =  (float)ND_pos(v)[i];
 	}
 	nsize[j].x = ND_width(v);
 	nsize[j].y = ND_height(v);
@@ -855,12 +782,12 @@ vpscAdjust(graph_t* G)
     opt.diredges = 0;
     opt.edge_gap = 0;
     opt.noverlap = 2;
-    opt.clusters = NEW(cluster_data);
-    margin = sepFactor (G);
+    opt.clusters = gv_alloc(sizeof(cluster_data));
+    exp_margin = sepFactor (G);
  	/* Multiply by 2 since opt.gap is the gap size, not the margin */
-    if (margin.doAdd) {
-	opt.gap.x = 2.0*PS2INCH(margin.x);
-	opt.gap.y = 2.0*PS2INCH(margin.y);
+    if (exp_margin.doAdd) {
+	opt.gap.x = 2.0*PS2INCH(exp_margin.x);
+	opt.gap.y = 2.0*PS2INCH(exp_margin.y);
     }
     else {
 	opt.gap.x = opt.gap.y = 2.0*PS2INCH(DFLT_MARGIN);
@@ -879,7 +806,6 @@ vpscAdjust(graph_t* G)
 
     free (opt.clusters);
     free (f_storage);
-    free (coords);
     free (nsize);
     return 0;
 }
@@ -897,7 +823,7 @@ angleSet (graph_t* g, double* phi)
     char* p;
     char* a = agget(g, "normalize");
 
-    if (!a || (*a == '\0'))
+    if (!a || *a == '\0')
 	return 0;
     ang = strtod (a, &p);
     if (p == a) {  /* no number */
@@ -982,7 +908,7 @@ typedef struct {
  * adjustMode[0] corresponds to overlap=true
  * adjustMode[1] corresponds to overlap=false
  */
-static lookup_t adjustMode[] = {
+static const lookup_t adjustMode[] = {
     ITEM(AM_NONE, "", "none"),
 #if ((defined(HAVE_GTS) || defined(HAVE_TRIANGLE)) && defined(SFDP))
     ITEM(AM_PRISM, "prism", "prism"),
@@ -1017,7 +943,7 @@ setPrismValues (Agraph_t* g, char* s, adjust_data* dp)
 {
     int v;
 
-    if ((sscanf (s, "%d", &v) > 0) && (v >= 0))
+    if (sscanf (s, "%d", &v) > 0 && v >= 0)
 	dp->value = v;
     else
 	dp->value = 1000;
@@ -1030,8 +956,8 @@ setPrismValues (Agraph_t* g, char* s, adjust_data* dp)
  */
 static adjust_data *getAdjustMode(Agraph_t* g, char *s, adjust_data* dp)
 {
-    lookup_t *ap = adjustMode + 1;
-    if ((s == NULL) || (*s == '\0')) {
+    const lookup_t *ap = adjustMode + 1;
+    if (s == NULL || *s == '\0') {
 	dp->mode = adjustMode[0].mode;
 	dp->print = adjustMode[0].print;
     }
@@ -1051,10 +977,11 @@ static adjust_data *getAdjustMode(Agraph_t* g, char *s, adjust_data* dp)
 	    ap++;
 	}
 	if (ap->attrib == NULL ) {
-	    int v = mapBool(s,'?');
-	    if (v == '?') {
+	    bool v = mapBool(s, false);
+	    bool unmappable = v != mapBool(s, true);
+	    if (unmappable) {
 		agerr (AGWARN, "Unrecognized overlap value \"%s\" - using false\n", s);
-		v = FALSE;
+		v = false;
 	    }
 	    if (v) {
 		dp->mode = adjustMode[0].mode;
@@ -1077,10 +1004,10 @@ static adjust_data *getAdjustMode(Agraph_t* g, char *s, adjust_data* dp)
 adjust_data *graphAdjustMode(graph_t *G, adjust_data* dp, char* dflt)
 {
     char* am = agget(G, "overlap");
-    return (getAdjustMode (G, am ? am : (dflt ? dflt : ""), dp));
+    return getAdjustMode (G, am ? am : (dflt ? dflt : ""), dp);
 }
 
-#define ISZERO(d) ((fabs(d) < 0.000000001))
+#define ISZERO(d) (fabs(d) < 0.000000001)
 
 /* simpleScaling:
  */
@@ -1096,7 +1023,7 @@ static int simpleScale (graph_t* g)
 	    if (ISZERO(sc.x)) return 0;
 	    if (i == 1) sc.y = sc.x;
 	    else if (ISZERO(sc.y)) return 0;
-	    if ((sc.y == 1) && (sc.x == 1)) return 0;
+	    if (sc.y == 1 && sc.x == 1) return 0;
 	    if (Verbose)
 		fprintf (stderr, "scale = (%.03f,%.03f)\n", sc.x, sc.y);
 	    for (n = agfstnode(g); n; n = agnxtnode(g,n)) {
@@ -1132,7 +1059,6 @@ removeOverlapWith (graph_t * G, adjust_data* am)
 	fprintf(stderr, "Adjusting %s using %s\n", agnameof(G), am->print);
 
     if (am->mode > AM_SCALE) {
-/* start_timer(); */
 	switch (am->mode) {
 	case AM_NSCALE:
 	    ret = scAdjust(G, 1);
@@ -1141,11 +1067,9 @@ removeOverlapWith (graph_t * G, adjust_data* am)
 	    ret = scAdjust(G, 0);
 	    break;
 	case AM_PUSH:
-	    /* scanAdjust (G, 1); */
         ret = 0;
 	    break;
 	case AM_PUSHPULL:
-	    /* scanAdjust (G, 0); */
         ret = 0;
 	    break;
 	case AM_PORTHO_YX:
@@ -1176,17 +1100,15 @@ removeOverlapWith (graph_t * G, adjust_data* am)
 	    break;
 #endif
 	default:		/* to silence warnings */
-	    if ((am->mode != AM_VOR) && (am->mode != AM_SCALE))
+	    if (am->mode != AM_VOR && am->mode != AM_SCALE)
 		agerr(AGWARN, "Unhandled adjust option %s\n", am->print);
 	    ret = 0;
 	    break;
 	}
-/* fprintf (stderr, "%s %.4f sec\n", am->print, elapsed_sec()); */
 	return nret+ret;
     }
 
     /* create main array */
-/* start_timer(); */
     if (makeInfo(G)) {
 	freeNodes();
 	free(sites);
@@ -1203,12 +1125,11 @@ removeOverlapWith (graph_t * G, adjust_data* am)
 	ret = vAdjust();
 
     if (ret)
-	updateGraph(G);
+	updateGraph();
 
     freeNodes();
     free(sites);
     sites = 0;
-/* fprintf (stderr, "%s %.4f sec\n", am->print, elapsed_sec()); */
 
     return ret+nret;
 }
@@ -1234,7 +1155,7 @@ removeOverlapAs(graph_t * G, char* flag)
  */
 int adjustNodes(graph_t * G)
 {
-    return (removeOverlapAs(G, agget(G, "overlap")));
+    return removeOverlapAs(G, agget(G, "overlap"));
 }
 
 /* parseFactor:
@@ -1249,23 +1170,23 @@ parseFactor (char* s, expand_t* pp, float sepfact, float dflt)
     int i;
     float x, y;
 
-    while (isspace(*s)) s++;
+    while (isspace((int)*s)) s++;
     if (*s == '+') {
 	s++;
-	pp->doAdd = 1;
+	pp->doAdd = true;
     }
-    else pp->doAdd = 0;
+    else pp->doAdd = false;
 
     if ((i = sscanf(s, "%f,%f", &x, &y))) {
 	if (i == 1) y = x;
 	if (pp->doAdd) {
 	    if (sepfact > 1) {
-		pp->x = MIN(dflt,x/sepfact);
-		pp->y = MIN(dflt,y/sepfact);
+		pp->x = fminf(dflt, x/sepfact);
+		pp->y = fminf(dflt, y/sepfact);
 	    }
 	    else if (sepfact < 1) {
-		pp->x = MAX(dflt,x/sepfact);
-		pp->y = MAX(dflt,y/sepfact);
+		pp->x = fmaxf(dflt, x/sepfact);
+		pp->y = fmaxf(dflt, y/sepfact);
 	    }
 	    else {
 		pp->x = x;
@@ -1273,8 +1194,8 @@ parseFactor (char* s, expand_t* pp, float sepfact, float dflt)
 	    }
 	}
 	else {
-	    pp->x = 1.0 + x/sepfact;
-	    pp->y = 1.0 + y/sepfact;
+	    pp->x = 1.0f + x/sepfact;
+	    pp->y = 1.0f + y/sepfact;
 	}
 	return 1;
     }
@@ -1295,7 +1216,7 @@ sepFactor(graph_t* g)
     }
     else { /* default */
 	pmargin.x = pmargin.y = DFLT_MARGIN;
-	pmargin.doAdd = 1;
+	pmargin.doAdd = true;
     }
     if (Verbose)
 	fprintf (stderr, "Node separation: add=%d (%f,%f)\n",
@@ -1317,11 +1238,11 @@ esepFactor(graph_t* g)
 
     if ((marg = agget(g, "esep")) && parseFactor(marg, &pmargin, 1.0, 0)) {
     }
-    else if ((marg = agget(g, "sep")) && parseFactor(marg, &pmargin, 1.0/SEPFACT, SEPFACT*DFLT_MARGIN)) {
+    else if ((marg = agget(g, "sep")) && parseFactor(marg, &pmargin, 1.0f/SEPFACT, SEPFACT*DFLT_MARGIN)) {
     }
     else {
 	pmargin.x = pmargin.y = SEPFACT*DFLT_MARGIN;
-	pmargin.doAdd = 1;
+	pmargin.doAdd = true;
     }
     if (Verbose)
 	fprintf (stderr, "Edge separation: add=%d (%f,%f)\n",
