@@ -20,6 +20,7 @@
 
 #include "config.h"
 
+#include <math.h>
 #include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
@@ -30,6 +31,7 @@
 
 #include <gvc/gvplugin_render.h>
 #include <cgraph/agxbuf.h>
+#include <cgraph/unreachable.h>
 #include <common/utils.h>
 #include <gvc/gvplugin_device.h>
 #include <gvc/gvio.h>
@@ -42,15 +44,16 @@
   #define EDGEALIGN 0
 #endif
 
-// optimized gvputs for string literals
-#define GVPUTS(job, str) gvwrite((job), (str), sizeof(str) - 1)
-
 typedef enum { FORMAT_SVG, FORMAT_SVGZ, } format_type;
 
 /* SVG dash array */
-static char *sdasharray = "5,2";
+static const char sdasharray[] = "5,2";
 /* SVG dot array */
-static char *sdotarray = "1,5";
+static const char sdotarray[] = "1,5";
+
+static const char transparent[] = "transparent";
+static const char none[] = "none";
+static const char black[] = "black";
 
 static void svg_bzptarray(GVJ_t * job, pointf * A, int n)
 {
@@ -64,7 +67,7 @@ static void svg_bzptarray(GVJ_t * job, pointf * A, int n)
 	for (i = 0; i < n; i++) {
 	    gvwrite(job, &c, 1);
             gvprintdouble(job, A[i].x);
-            GVPUTS(job, ",");
+            gvputc(job, ',');
             gvprintdouble(job, -A[i].y);
 	    if (i == 0)
 		c = 'C';		/* second point */
@@ -76,7 +79,7 @@ static void svg_bzptarray(GVJ_t * job, pointf * A, int n)
 	for (i = n-1; i >= 0; i--) {
 	    gvwrite(job, &c, 1);
             gvprintdouble(job, A[i].x);
-            GVPUTS(job, ",");
+            gvputc(job, ',');
             gvprintdouble(job, -A[i].y);
 	    if (i == 0)
 		c = 'C';		/* second point */
@@ -91,35 +94,66 @@ static void svg_print_id_class(GVJ_t * job, char* id, char* idx, char* kind, voi
 {
     char* str;
 
-    GVPUTS(job, "<g id=\"");
+    gvputs(job, "<g id=\"");
     gvputs_xml(job, id);
     if (idx) {
-	GVPUTS(job, "_");
+	gvputc(job, '_');
 	gvputs_xml(job, idx);
     }
     gvprintf(job, "\" class=\"%s", kind);
     if ((str = agget(obj, "class")) && *str) {
-	GVPUTS(job, " ");
+	gvputc(job, ' ');
 	gvputs_xml(job, str);
     }
-    GVPUTS(job, "\"");
+    gvputc(job, '"');
 }
 
-static void svg_print_color(GVJ_t * job, gvcolor_t color)
+/* svg_print_paint assumes the caller will set the opacity if the alpha channel
+ * is greater than 0 and less than 255
+ */
+static void svg_print_paint(GVJ_t * job, gvcolor_t color)
 {
     switch (color.type) {
     case COLOR_STRING:
-	gvputs(job, color.u.string);
+	if (!strcmp(color.u.string, transparent))
+	    gvputs(job, none);
+	else
+	    gvputs(job, color.u.string);
 	break;
     case RGBA_BYTE:
 	if (color.u.rgba[3] == 0)	/* transparent */
-	    GVPUTS(job, "transparent");
+	    gvputs(job, none);
 	else
 	    gvprintf(job, "#%02x%02x%02x",
 		     color.u.rgba[0], color.u.rgba[1], color.u.rgba[2]);
 	break;
     default:
-	assert(0);		/* internal error */
+	UNREACHABLE(); // internal error
+    }
+}
+
+/* svg_print_gradient_color assumes the caller will set the opacity if the
+ * alpha channel is less than 255.
+ *
+ * "transparent" in SVG 2 gradients is considered to be black with 0 opacity,
+ * so for compatibility with SVG 1.1 output we use black when the color string
+ * is transparent and assume the caller will also check and set opacity 0.
+ */
+static void svg_print_gradient_color(GVJ_t * job, gvcolor_t color)
+{
+    switch (color.type) {
+    case COLOR_STRING:
+	if (!strcmp(color.u.string, transparent))
+	    gvputs(job, black);
+	else
+	    gvputs(job, color.u.string);
+	break;
+    case RGBA_BYTE:
+	gvprintf(job, "#%02x%02x%02x",
+		 color.u.rgba[0], color.u.rgba[1], color.u.rgba[2]);
+	break;
+    default:
+	UNREACHABLE(); // internal error
     }
 }
 
@@ -127,25 +161,37 @@ static void svg_grstyle(GVJ_t * job, int filled, int gid)
 {
     obj_state_t *obj = job->obj;
 
-    GVPUTS(job, " fill=\"");
+    gvputs(job, " fill=\"");
     if (filled == GRADIENT) {
-	gvprintf(job, "url(#l_%d)", gid);
+	gvputs(job, "url(#");
+	if (obj->id != NULL) {
+	    gvputs_xml(job, obj->id);
+	    gvputc(job, '_');
+	}
+	gvprintf(job, "l_%d)", gid);
     } else if (filled == RGRADIENT) {
-	gvprintf(job, "url(#r_%d)", gid);
+	gvputs(job, "url(#");
+	if (obj->id != NULL) {
+	    gvputs_xml(job, obj->id);
+	    gvputc(job, '_');
+	}
+	gvprintf(job, "r_%d)", gid);
     } else if (filled) {
-	svg_print_color(job, obj->fillcolor);
+	svg_print_paint(job, obj->fillcolor);
 	if (obj->fillcolor.type == RGBA_BYTE
 	    && obj->fillcolor.u.rgba[3] > 0
 	    && obj->fillcolor.u.rgba[3] < 255)
 	    gvprintf(job, "\" fill-opacity=\"%f",
-		     ((float) obj->fillcolor.u.rgba[3] / 255.0));
+		     (float)obj->fillcolor.u.rgba[3] / 255.0);
     } else {
-	GVPUTS(job, "none");
+	gvputs(job, "none");
     }
-    GVPUTS(job, "\" stroke=\"");
-    svg_print_color(job, obj->pencolor);
-    if (obj->penwidth != PENWIDTH_NORMAL) {
-	GVPUTS(job, "\" stroke-width=\"");
+    gvputs(job, "\" stroke=\"");
+    svg_print_paint(job, obj->pencolor);
+    // will `gvprintdouble` output something different from `PENWIDTH_NORMAL`?
+    const double GVPRINT_DOUBLE_THRESHOLD = 0.005;
+    if (!(fabs(obj->penwidth - PENWIDTH_NORMAL) < GVPRINT_DOUBLE_THRESHOLD)) {
+	gvputs(job, "\" stroke-width=\"");
         gvprintdouble(job, obj->penwidth);
     }
     if (obj->pen == PEN_DASHED) {
@@ -156,38 +202,38 @@ static void svg_grstyle(GVJ_t * job, int filled, int gid)
     if (obj->pencolor.type == RGBA_BYTE && obj->pencolor.u.rgba[3] > 0
 	&& obj->pencolor.u.rgba[3] < 255)
 	gvprintf(job, "\" stroke-opacity=\"%f",
-		 ((float) obj->pencolor.u.rgba[3] / 255.0));
+		 (float)obj->pencolor.u.rgba[3] / 255.0);
 
-    GVPUTS(job, "\"");
+    gvputc(job, '"');
 }
 
 static void svg_comment(GVJ_t * job, char *str)
 {
-    GVPUTS(job, "<!-- ");
+    gvputs(job, "<!-- ");
     gvputs_xml(job, str);
-    GVPUTS(job, " -->\n");
+    gvputs(job, " -->\n");
 }
 
 static void svg_begin_job(GVJ_t * job)
 {
     char *s;
-    GVPUTS(job,
+    gvputs(job,
 	   "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>\n");
     if ((s = agget(job->gvc->g, "stylesheet")) && s[0]) {
-	GVPUTS(job, "<?xml-stylesheet href=\"");
+	gvputs(job, "<?xml-stylesheet href=\"");
 	gvputs(job, s);
-	GVPUTS(job, "\" type=\"text/css\"?>\n");
+	gvputs(job, "\" type=\"text/css\"?>\n");
     }
-    GVPUTS(job, "<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.1//EN\"\n"
+    gvputs(job, "<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.1//EN\"\n"
                 " \"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd\">\n"
 
                 "<!-- Generated by ");
     gvputs_xml(job, job->common->info[0]);
-    GVPUTS(job, " version ");
+    gvputs(job, " version ");
     gvputs_xml(job, job->common->info[1]);
-    GVPUTS(job, " (");
+    gvputs(job, " (");
     gvputs_xml(job, job->common->info[2]);
-    GVPUTS(job, ")\n"
+    gvputs(job, ")\n"
                 " -->\n");
 }
 
@@ -195,9 +241,9 @@ static void svg_begin_graph(GVJ_t * job)
 {
     obj_state_t *obj = job->obj;
 
-    GVPUTS(job, "<!--");
+    gvputs(job, "<!--");
     if (agnameof(obj->u.g)[0] && agnameof(obj->u.g)[0] != LOCALNAMEPREFIX) {
-	GVPUTS(job, " Title: ");
+	gvputs(job, " Title: ");
 	gvputs_xml(job, agnameof(obj->u.g));
     }
     gvprintf(job, " Pages: %d -->\n",
@@ -211,7 +257,7 @@ static void svg_begin_graph(GVJ_t * job)
 	job->canvasBox.UR.x,
 	job->canvasBox.UR.y);
     /* namespace of svg */
-    GVPUTS(job, " xmlns=\"http://www.w3.org/2000/svg\""
+    gvputs(job, " xmlns=\"http://www.w3.org/2000/svg\""
     /* namespace of xlink */
                 " xmlns:xlink=\"http://www.w3.org/1999/xlink\""
                 ">\n");
@@ -219,7 +265,7 @@ static void svg_begin_graph(GVJ_t * job)
 
 static void svg_end_graph(GVJ_t * job)
 {
-    GVPUTS(job, "</svg>\n");
+    gvputs(job, "</svg>\n");
 }
 
 static void svg_begin_layer(GVJ_t * job, char *layername, int layerNum,
@@ -231,12 +277,12 @@ static void svg_begin_layer(GVJ_t * job, char *layername, int layerNum,
     obj_state_t *obj = job->obj;
 
     svg_print_id_class(job, layername, NULL, "layer", obj->u.g);
-    GVPUTS(job, ">\n");
+    gvputs(job, ">\n");
 }
 
 static void svg_end_layer(GVJ_t * job)
 {
-    GVPUTS(job, "</g>\n");
+    gvputs(job, "</g>\n");
 }
 
 /* svg_begin_page:
@@ -250,25 +296,25 @@ static void svg_begin_page(GVJ_t * job)
     /* its really just a page of the graph, but its still a graph,
      * and it is the entire graph if we're not currently paging */
     svg_print_id_class(job, obj->id, NULL, "graph", obj->u.g);
-    GVPUTS(job, " transform=\"scale(");
+    gvputs(job, " transform=\"scale(");
     // cannot be gvprintdouble because 2 digits precision insufficient
     gvprintf(job, "%g %g", job->scale.x, job->scale.y);
     gvprintf(job, ") rotate(%d) translate(", -job->rotation);
     gvprintdouble(job, job->translation.x);
-    GVPUTS(job, " ");
+    gvputc(job, ' ');
     gvprintdouble(job, -job->translation.y);
-    GVPUTS(job, ")\">\n");
+    gvputs(job, ")\">\n");
     /* default style */
     if (agnameof(obj->u.g)[0] && agnameof(obj->u.g)[0] != LOCALNAMEPREFIX) {
-	GVPUTS(job, "<title>");
+	gvputs(job, "<title>");
 	gvputs_xml(job, agnameof(obj->u.g));
-	GVPUTS(job, "</title>\n");
+	gvputs(job, "</title>\n");
     }
 }
 
 static void svg_end_page(GVJ_t * job)
 {
-    GVPUTS(job, "</g>\n");
+    gvputs(job, "</g>\n");
 }
 
 static void svg_begin_cluster(GVJ_t * job)
@@ -276,15 +322,15 @@ static void svg_begin_cluster(GVJ_t * job)
     obj_state_t *obj = job->obj;
 
     svg_print_id_class(job, obj->id, NULL, "cluster", obj->u.sg);
-    GVPUTS(job, ">\n"
+    gvputs(job, ">\n"
                 "<title>");
     gvputs_xml(job, agnameof(obj->u.g));
-    GVPUTS(job, "</title>\n");
+    gvputs(job, "</title>\n");
 }
 
 static void svg_end_cluster(GVJ_t * job)
 {
-    GVPUTS(job, "</g>\n");
+    gvputs(job, "</g>\n");
 }
 
 static void svg_begin_node(GVJ_t * job)
@@ -297,15 +343,15 @@ static void svg_begin_node(GVJ_t * job)
     else
 	idx = NULL;
     svg_print_id_class(job, obj->id, idx, "node", obj->u.n);
-    GVPUTS(job, ">\n"
+    gvputs(job, ">\n"
                 "<title>");
     gvputs_xml(job, agnameof(obj->u.n));
-    GVPUTS(job, "</title>\n");
+    gvputs(job, "</title>\n");
 }
 
 static void svg_end_node(GVJ_t * job)
 {
-    GVPUTS(job, "</g>\n");
+    gvputs(job, "</g>\n");
 }
 
 static void svg_begin_edge(GVJ_t * job)
@@ -314,56 +360,56 @@ static void svg_begin_edge(GVJ_t * job)
     char *ename;
 
     svg_print_id_class(job, obj->id, NULL, "edge", obj->u.e);
-    GVPUTS(job, ">\n"
+    gvputs(job, ">\n"
 
                 "<title>");
     ename = strdup_and_subst_obj("\\E", obj->u.e);
     gvputs_xml(job, ename);
     free(ename);
-    GVPUTS(job, "</title>\n");
+    gvputs(job, "</title>\n");
 }
 
 static void svg_end_edge(GVJ_t * job)
 {
-    GVPUTS(job, "</g>\n");
+    gvputs(job, "</g>\n");
 }
 
 static void
 svg_begin_anchor(GVJ_t * job, char *href, char *tooltip, char *target,
 		 char *id)
 {
-    GVPUTS(job, "<g");
+    gvputs(job, "<g");
     if (id) {
-	GVPUTS(job, " id=\"a_");
+	gvputs(job, " id=\"a_");
         gvputs_xml(job, id);
-        GVPUTS(job, "\"");
+        gvputc(job, '"');
     }
-    GVPUTS(job, ">"
+    gvputs(job, ">"
 
                 "<a");
     if (href && href[0]) {
-	GVPUTS(job, " xlink:href=\"");
+	gvputs(job, " xlink:href=\"");
 	const xml_flags_t flags = {0};
 	xml_escape(href, flags, (int(*)(void*, const char*))gvputs, job);
-	GVPUTS(job, "\"");
+	gvputc(job, '"');
     }
     if (tooltip && tooltip[0]) {
-	GVPUTS(job, " xlink:title=\"");
+	gvputs(job, " xlink:title=\"");
 	const xml_flags_t flags = {.raw = 1, .dash = 1, .nbsp = 1};
 	xml_escape(tooltip, flags, (int(*)(void*, const char*))gvputs, job);
-	GVPUTS(job, "\"");
+	gvputc(job, '"');
     }
     if (target && target[0]) {
-	GVPUTS(job, " target=\"");
+	gvputs(job, " target=\"");
 	gvputs_xml(job, target);
-	GVPUTS(job, "\"");
+	gvputc(job, '"');
     }
-    GVPUTS(job, ">\n");
+    gvputs(job, ">\n");
 }
 
 static void svg_end_anchor(GVJ_t * job)
 {
-    GVPUTS(job, "</a>\n"
+    gvputs(job, "</a>\n"
                 "</g>\n");
 }
 
@@ -374,26 +420,26 @@ static void svg_textspan(GVJ_t * job, pointf p, textspan_t * span)
     char *family = NULL, *weight = NULL, *stretch = NULL, *style = NULL;
     unsigned int flags;
 
-    GVPUTS(job, "<text");
+    gvputs(job, "<text");
     switch (span->just) {
     case 'l':
-	GVPUTS(job, " text-anchor=\"start\"");
+	gvputs(job, " text-anchor=\"start\"");
 	break;
     case 'r':
-	GVPUTS(job, " text-anchor=\"end\"");
+	gvputs(job, " text-anchor=\"end\"");
 	break;
     default:
     case 'n':
-	GVPUTS(job, " text-anchor=\"middle\"");
+	gvputs(job, " text-anchor=\"middle\"");
 	break;
     }
     p.y += span->yoffset_centerline;
     if (!obj->labeledgealigned) {
-	GVPUTS(job, " x=\"");
+	gvputs(job, " x=\"");
         gvprintdouble(job, p.x);
-        GVPUTS(job, "\" y=\"");
+        gvputs(job, "\" y=\"");
         gvprintdouble(job, -p.y);
-        GVPUTS(job, "\"");
+        gvputs(job, "\"");
     }
     pA = span->font->postscript_alias;
     if (pA) {
@@ -420,7 +466,7 @@ static void svg_textspan(GVJ_t * job, pointf p, textspan_t * span)
 	gvprintf(job, " font-family=\"%s", family);
 	if (pA->svg_font_family)
 	    gvprintf(job, ",%s", pA->svg_font_family);
-	GVPUTS(job, "\"");
+	gvputc(job, '"');
 	if (weight)
 	    gvprintf(job, " font-weight=\"%s\"", weight);
 	if (stretch)
@@ -429,30 +475,30 @@ static void svg_textspan(GVJ_t * job, pointf p, textspan_t * span)
 	    gvprintf(job, " font-style=\"%s\"", style);
     } else
 	gvprintf(job, " font-family=\"%s\"", span->font->name);
-    if ((span->font) && (flags = span->font->flags)) {
+    if (span->font && (flags = span->font->flags)) {
 	if ((flags & HTML_BF) && !weight)
-	    GVPUTS(job, " font-weight=\"bold\"");
+	    gvputs(job, " font-weight=\"bold\"");
 	if ((flags & HTML_IF) && !style)
-	    GVPUTS(job, " font-style=\"italic\"");
-	if ((flags & (HTML_UL|HTML_S|HTML_OL))) {
+	    gvputs(job, " font-style=\"italic\"");
+	if (flags & (HTML_UL|HTML_S|HTML_OL)) {
 	    int comma = 0;
-	    GVPUTS(job, " text-decoration=\"");
+	    gvputs(job, " text-decoration=\"");
 	    if ((flags & HTML_UL)) {
-		GVPUTS(job, "underline");
+		gvputs(job, "underline");
 		comma = 1;
 	    }
-	    if ((flags & HTML_OL)) {
+	    if (flags & HTML_OL) {
 		gvprintf(job, "%soverline", (comma?",":""));
 		comma = 1;
 	    }
-	    if ((flags & HTML_S))
+	    if (flags & HTML_S)
 		gvprintf(job, "%sline-through", (comma?",":""));
-	    GVPUTS(job, "\"");
+	    gvputc(job, '"');
 	}
-	if ((flags & HTML_SUP))
-	    GVPUTS(job, " baseline-shift=\"super\"");
-	if ((flags & HTML_SUB))
-	    GVPUTS(job, " baseline-shift=\"sub\"");
+	if (flags & HTML_SUP)
+	    gvputs(job, " baseline-shift=\"super\"");
+	if (flags & HTML_SUB)
+	    gvputs(job, " baseline-shift=\"sub\"");
     }
 
     gvprintf(job, " font-size=\"%.2f\"", span->font->size);
@@ -465,25 +511,44 @@ static void svg_textspan(GVJ_t * job, pointf p, textspan_t * span)
 	gvprintf(job, " fill=\"#%02x%02x%02x\"",
 		 obj->pencolor.u.rgba[0], obj->pencolor.u.rgba[1],
 		 obj->pencolor.u.rgba[2]);
-	if (obj->pencolor.u.rgba[3] > 0 && obj->pencolor.u.rgba[3] < 255)
-	    gvprintf(job, " fill-opacity=\"%f\"", ((float) obj->pencolor.u.rgba[3] / 255.0));
+	if (obj->pencolor.u.rgba[3] < 255)
+	    gvprintf(job, " fill-opacity=\"%f\"", (float)obj->pencolor.u.rgba[3] / 255.0);
 	break;
     default:
-	assert(0);		/* internal error */
+	UNREACHABLE(); // internal error
     }
-    GVPUTS(job, ">");
+    gvputc(job, '>');
     if (obj->labeledgealigned) {
-	GVPUTS(job, "<textPath xlink:href=\"#");
+	gvputs(job, "<textPath xlink:href=\"#");
 	gvputs_xml(job, obj->id);
-	GVPUTS(job, "_p\" startOffset=\"50%\"><tspan x=\"0\" dy=\"");
+	gvputs(job, "_p\" startOffset=\"50%\"><tspan x=\"0\" dy=\"");
         gvprintdouble(job, -p.y);
-        GVPUTS(job, "\">");
+        gvputs(job, "\">");
     }
     const xml_flags_t xml_flags = {.raw = 1, .dash = 1, .nbsp = 1};
     xml_escape(span->str, xml_flags, (int(*)(void*, const char*))gvputs, job);
     if (obj->labeledgealigned)
-	GVPUTS(job, "</tspan></textPath>");
-    GVPUTS(job, "</text>\n");
+	gvputs(job, "</tspan></textPath>");
+    gvputs(job, "</text>\n");
+}
+
+static void svg_print_stop(GVJ_t * job, double offset, gvcolor_t color)
+{
+    if (fabs(offset - 0.0) < 0.0005)
+	gvputs(job, "<stop offset=\"0\" style=\"stop-color:");
+    else if (fabs(offset - 1.0) < 0.0005)
+	gvputs(job, "<stop offset=\"1\" style=\"stop-color:");
+    else
+	gvprintf(job, "<stop offset=\"%.03f\" style=\"stop-color:", offset);
+    svg_print_gradient_color(job, color);
+    gvputs(job, ";stop-opacity:");
+    if (color.type == RGBA_BYTE && color.u.rgba[3] < 255)
+	gvprintf(job, "%f", (float)color.u.rgba[3] / 255.0);
+    else if (color.type == COLOR_STRING && !strcmp(color.u.string, transparent))
+	gvputs(job, "0");
+    else
+	gvputs(job, "1.");
+    gvputs(job, ";\"/>\n");
 }
 
 /* svg_gradstyle
@@ -500,41 +565,26 @@ static int svg_gradstyle(GVJ_t * job, pointf * A, int n)
     G[0].x = G[0].y = G[1].x = G[1].y = 0.;
     get_gradient_points(A, G, n, angle, 0);	//get points on gradient line
 
-    gvprintf(job,
-	     "<defs>\n<linearGradient id=\"l_%d\" gradientUnits=\"userSpaceOnUse\" ", id);
-    GVPUTS(job, "x1=\"");
+    gvputs(job, "<defs>\n<linearGradient id=\"");
+    if (obj->id != NULL) {
+        gvputs_xml(job, obj->id);
+        gvputc(job, '_');
+    }
+    gvprintf(job, "l_%d\" gradientUnits=\"userSpaceOnUse\" ", id);
+    gvputs(job, "x1=\"");
     gvprintdouble(job, G[0].x);
-    GVPUTS(job, "\" y1=\"");
+    gvputs(job, "\" y1=\"");
     gvprintdouble(job, G[0].y);
-    GVPUTS(job, "\" x2=\"");
+    gvputs(job, "\" x2=\"");
     gvprintdouble(job, G[1].x);
-    GVPUTS(job, "\" y2=\"");
+    gvputs(job, "\" y2=\"");
     gvprintdouble(job, G[1].y);
-    GVPUTS(job, "\" >\n");
-    if (obj->gradient_frac > 0)
-	gvprintf(job, "<stop offset=\"%.03f\" style=\"stop-color:", obj->gradient_frac - 0.001);
-    else
-	GVPUTS(job, "<stop offset=\"0\" style=\"stop-color:");
-    svg_print_color(job, obj->fillcolor);
-    GVPUTS(job, ";stop-opacity:");
-    if (obj->fillcolor.type == RGBA_BYTE && obj->fillcolor.u.rgba[3] > 0
-	&& obj->fillcolor.u.rgba[3] < 255)
-	gvprintf(job, "%f", ((float) obj->fillcolor.u.rgba[3] / 255.0));
-    else
-	GVPUTS(job, "1.");
-    GVPUTS(job, ";\"/>\n");
-    if (obj->gradient_frac > 0)
-	gvprintf(job, "<stop offset=\"%.03f\" style=\"stop-color:", obj->gradient_frac);
-    else
-	GVPUTS(job, "<stop offset=\"1\" style=\"stop-color:");
-    svg_print_color(job, obj->stopcolor);
-    GVPUTS(job, ";stop-opacity:");
-    if (obj->stopcolor.type == RGBA_BYTE && obj->stopcolor.u.rgba[3] > 0
-	&& obj->stopcolor.u.rgba[3] < 255)
-	gvprintf(job, "%f", ((float) obj->stopcolor.u.rgba[3] / 255.0));
-    else
-	GVPUTS(job, "1.");
-    GVPUTS(job, ";\"/>\n</linearGradient>\n</defs>\n");
+    gvputs(job, "\" >\n");
+
+    svg_print_stop(job, obj->gradient_frac > 0 ? obj->gradient_frac - 0.001 : 0.0, obj->fillcolor);
+    svg_print_stop(job, obj->gradient_frac > 0 ? obj->gradient_frac : 1.0, obj->stopcolor);
+
+    gvputs(job, "</linearGradient>\n</defs>\n");
     return id;
 }
 
@@ -543,40 +593,31 @@ static int svg_gradstyle(GVJ_t * job, pointf * A, int n)
  */
 static int svg_rgradstyle(GVJ_t * job)
 {
-    float angle;
-    int ifx, ify;
+    double ifx, ify;
     static int rgradId;
     int id = rgradId++;
 
     obj_state_t *obj = job->obj;
-    angle = obj->gradient_angle * M_PI / 180;	//angle of gradient line
-    if (angle == 0.) {
+    if (obj->gradient_angle == 0) {
 	ifx = ify = 50;
     } else {
-	ifx = 50 * (1 + cos(angle));
-	ify = 50 * (1 - sin(angle));
+	double angle = obj->gradient_angle * M_PI / 180;	//angle of gradient line
+	ifx = round(50 * (1 + cos(angle)));
+	ify = round(50 * (1 - sin(angle)));
     }
-    gvprintf(job,
-	     "<defs>\n<radialGradient id=\"r_%d\" cx=\"50%%\" cy=\"50%%\" r=\"75%%\" fx=\"%d%%\" fy=\"%d%%\">\n",
+    gvputs(job, "<defs>\n<radialGradient id=\"");
+    if (obj->id != NULL) {
+	gvputs_xml(job, obj->id);
+	gvputc(job, '_');
+    }
+    gvprintf(job, "r_%d\" cx=\"50%%\" cy=\"50%%\" r=\"75%%\" "
+	     "fx=\"%.0f%%\" fy=\"%.0f%%\">\n",
 	     id, ifx, ify);
-    GVPUTS(job, "<stop offset=\"0\" style=\"stop-color:");
-    svg_print_color(job, obj->fillcolor);
-    GVPUTS(job, ";stop-opacity:");
-    if (obj->fillcolor.type == RGBA_BYTE && obj->fillcolor.u.rgba[3] > 0
-	&& obj->fillcolor.u.rgba[3] < 255)
-	gvprintf(job, "%f", ((float) obj->fillcolor.u.rgba[3] / 255.0));
-    else
-	GVPUTS(job, "1.");
-    GVPUTS(job, ";\"/>\n"
-                "<stop offset=\"1\" style=\"stop-color:");
-    svg_print_color(job, obj->stopcolor);
-    GVPUTS(job, ";stop-opacity:");
-    if (obj->stopcolor.type == RGBA_BYTE && obj->stopcolor.u.rgba[3] > 0
-	&& obj->stopcolor.u.rgba[3] < 255)
-	gvprintf(job, "%f", ((float) obj->stopcolor.u.rgba[3] / 255.0));
-    else
-	GVPUTS(job, "1.");
-    GVPUTS(job, ";\"/>\n</radialGradient>\n</defs>\n");
+
+    svg_print_stop(job, 0.0, obj->fillcolor);
+    svg_print_stop(job, 1.0, obj->stopcolor);
+
+    gvputs(job, "</radialGradient>\n</defs>\n");
     return id;
 }
 
@@ -588,20 +629,20 @@ static void svg_ellipse(GVJ_t * job, pointf * A, int filled)
     /* A[] contains 2 points: the center and corner. */
     if (filled == GRADIENT) {
 	gid = svg_gradstyle(job, A, 2);
-    } else if (filled == (RGRADIENT)) {
+    } else if (filled == RGRADIENT) {
 	gid = svg_rgradstyle(job);
     }
-    GVPUTS(job, "<ellipse");
+    gvputs(job, "<ellipse");
     svg_grstyle(job, filled, gid);
-    GVPUTS(job, " cx=\"");
+    gvputs(job, " cx=\"");
     gvprintdouble(job, A[0].x);
-    GVPUTS(job, "\" cy=\"");
+    gvputs(job, "\" cy=\"");
     gvprintdouble(job, -A[0].y);
-    GVPUTS(job, "\" rx=\"");
+    gvputs(job, "\" rx=\"");
     gvprintdouble(job, A[1].x - A[0].x);
-    GVPUTS(job, "\" ry=\"");
+    gvputs(job, "\" ry=\"");
     gvprintdouble(job, A[1].y - A[0].y);
-    GVPUTS(job, "\"/>\n");
+    gvputs(job, "\"/>\n");
 }
 
 static void
@@ -616,19 +657,19 @@ svg_bezier(GVJ_t * job, pointf * A, int n, int arrow_at_start,
   
     if (filled == GRADIENT) {
 	gid = svg_gradstyle(job, A, n);
-    } else if (filled == (RGRADIENT)) {
+    } else if (filled == RGRADIENT) {
 	gid = svg_rgradstyle(job);
     }
-    GVPUTS(job, "<path");
+    gvputs(job, "<path");
     if (obj->labeledgealigned) {
-	GVPUTS(job, " id=\"");
+	gvputs(job, " id=\"");
 	gvputs_xml(job, obj->id);
-	GVPUTS(job, "_p\" ");
+	gvputs(job, "_p\" ");
     } 
     svg_grstyle(job, filled, gid);
-    GVPUTS(job, " d=\"");
+    gvputs(job, " d=\"");
     svg_bzptarray(job, A, n);
-    GVPUTS(job, "\"/>\n");
+    gvputs(job, "\"/>\n");
 }
 
 static void svg_polygon(GVJ_t * job, pointf * A, int n, int filled)
@@ -636,39 +677,41 @@ static void svg_polygon(GVJ_t * job, pointf * A, int n, int filled)
     int i, gid = 0;
     if (filled == GRADIENT) {
 	gid = svg_gradstyle(job, A, n);
-    } else if (filled == (RGRADIENT)) {
+    } else if (filled == RGRADIENT) {
 	gid = svg_rgradstyle(job);
     }
-    GVPUTS(job, "<polygon");
+    gvputs(job, "<polygon");
     svg_grstyle(job, filled, gid);
-    GVPUTS(job, " points=\"");
+    gvputs(job, " points=\"");
     for (i = 0; i < n; i++) {
         gvprintdouble(job, A[i].x);
-        GVPUTS(job, ",");
+        gvputc(job, ',');
         gvprintdouble(job, -A[i].y);
-        GVPUTS(job, " ");
+        gvputc(job, ' ');
     }
     /* repeat the first point because Adobe SVG is broken */
     gvprintdouble(job, A[0].x);
-    GVPUTS(job, ",");
+    gvputc(job, ',');
     gvprintdouble(job, -A[0].y);
-    GVPUTS(job, "\"/>\n");
+    gvputs(job, "\"/>\n");
 }
 
 static void svg_polyline(GVJ_t * job, pointf * A, int n)
 {
     int i;
 
-    GVPUTS(job, "<polyline");
+    gvputs(job, "<polyline");
     svg_grstyle(job, 0, 0);
-    GVPUTS(job, " points=\"");
+    gvputs(job, " points=\"");
     for (i = 0; i < n; i++) {
         gvprintdouble(job, A[i].x);
-        GVPUTS(job, ",");
+        gvputc(job, ',');
         gvprintdouble(job, -A[i].y);
-        GVPUTS(job, " ");
+        if (i != n - 1) {
+            gvputc(job, ' ');
+        }
     }
-    GVPUTS(job, "\"/>\n");
+    gvputs(job, "\"/>\n");
 }
 
 /* color names from http://www.w3.org/TR/SVG/types.html */
