@@ -10,7 +10,6 @@ import os
 from pathlib import Path
 import platform
 import re
-import shutil
 import signal
 import stat
 import subprocess
@@ -21,7 +20,7 @@ import xml.etree.ElementTree as ET
 import pytest
 
 sys.path.append(os.path.dirname(__file__))
-from gvtest import dot, gvpr, ROOT, remove_xtype_warnings, run_c \
+from gvtest import dot, gvpr, ROOT, remove_xtype_warnings, run_c, which \
   #pylint: disable=wrong-import-position
 
 def is_ndebug_defined() -> bool:
@@ -43,20 +42,11 @@ def is_ndebug_defined() -> bool:
 # saying "== Failed ==" which thus is not counted as a failure at the
 # end.
 
-def test_regression_subset_differences():
-  """
-  First run a subset of all the tests that produces equal output files
-  for all platforms and fail the test if there are differences.
-  """
-
-  os.chdir(Path(__file__).resolve().parent)
-  subprocess.check_call([sys.executable, "rtest.py", "tests_subset.txt"])
-
 def test_regression_failure():
   """
-  Secondly, run all tests but ignore differences and fail the test
-  only if there is a crash. This will leave the differences for png
-  output in tests/nhtml/index.html for review.
+  Run all tests but ignore differences and fail the test only if there is a
+  crash. This will leave the differences for png output in
+  tests/nhtml/index.html for review.
   """
 
   os.chdir(Path(__file__).resolve().parent)
@@ -84,6 +74,21 @@ def test_14():
   # process it with Graphviz
   dot("svg", input)
 
+@pytest.mark.skipif(which("neato") is None, reason="neato not available")
+def test_42():
+  """
+  check for a former crash in neatogen
+  https://gitlab.com/graphviz/graphviz/-/issues/42
+  """
+
+  # locate our associated test case in this directory
+  input = Path(__file__).parent / "42.dot"
+  assert input.exists(), "unexpectedly missing test case"
+
+  # process it with Graphviz
+  subprocess.check_call(["neato", "-n2", "-Tpng", input],
+                        stdout=subprocess.DEVNULL)
+
 def test_56():
   """
   parsing a particular graph should not cause a Trapezoid-table overflow
@@ -129,7 +134,7 @@ def test_131():
   # ask Graphviz to process this to PIC
   pic = dot("pic", source=src)
 
-  if shutil.which("gpic") is None:
+  if which("gpic") is None:
     pytest.skip("GNU PIC not available")
 
   # ask GNU PIC to process the Graphviz output
@@ -325,8 +330,7 @@ def test_358():
     assert m is not None, \
       f"font characteristic {1 << i} not enabled in xdot 1.7"
 
-@pytest.mark.skipif(shutil.which("gv2gxl") is None or
-                    shutil.which("gxl2gv") is None,
+@pytest.mark.skipif(which("gv2gxl") is None or which("gxl2gv") is None,
                     reason="GXL tools not available")
 def test_517():
   """
@@ -438,8 +442,7 @@ def test_1221():
   # process this with dot
   dot("svg", input)
 
-@pytest.mark.skipif(shutil.which("gv2gml") is None,
-                    reason="gv2gml not available")
+@pytest.mark.skipif(which("gv2gml") is None, reason="gv2gml not available")
 def test_1276():
   """
   quotes within a label should be escaped in translation to GML
@@ -583,7 +586,7 @@ def test_1449():
 
   assert stderr.strip() == "", "SVG color scheme use caused warnings"
 
-@pytest.mark.skipif(shutil.which("gvpr") is None, reason="GVPR not available")
+@pytest.mark.skipif(which("gvpr") is None, reason="GVPR not available")
 def test_1594():
   """
   GVPR should give accurate line numbers in error messages
@@ -694,7 +697,7 @@ def test_1767():
   #                  "cluster_2 contains 3 nodes\n" \
   #                  "cluster_3 contains 3 nodes\n"
 
-@pytest.mark.skipif(shutil.which("gvpr") is None, reason="GVPR not available")
+@pytest.mark.skipif(which("gvpr") is None, reason="GVPR not available")
 @pytest.mark.skipif(platform.system() != "Windows",
   reason="only relevant on Windows")
 def test_1780():
@@ -726,7 +729,7 @@ def test_1783():
 
   assert ret != -signal.SIGSEGV, "Graphviz segfaulted"
 
-@pytest.mark.skipif(shutil.which("gvedit") is None, reason="Gvedit not available")
+@pytest.mark.skipif(which("gvedit") is None, reason="Gvedit not available")
 def test_1813():
   """
   gvedit -? should show usage
@@ -806,7 +809,7 @@ def test_1856():
 
   assert all(y >= top_of_five for y in waypoints_y), "edge dips below 5"
 
-@pytest.mark.skipif(shutil.which("fdp") is None, reason="fdp not available")
+@pytest.mark.skipif(which("fdp") is None, reason="fdp not available")
 def test_1865():
   """
   fdp should not read out of bounds when processing node names
@@ -822,10 +825,8 @@ def test_1865():
   # fdp should not crash when processing this file
   subprocess.check_call(["fdp", "-o", os.devnull, input])
 
-@pytest.mark.skipif(shutil.which("gv2gml") is None,
-                    reason="gv2gml not available")
-@pytest.mark.skipif(shutil.which("gml2gv") is None,
-                    reason="gml2gv not available")
+@pytest.mark.skipif(which("gv2gml") is None, reason="gv2gml not available")
+@pytest.mark.skipif(which("gml2gv") is None, reason="gml2gv not available")
 @pytest.mark.parametrize("penwidth", ("1.0", "1"))
 def test_1871(penwidth: str):
   """
@@ -848,7 +849,7 @@ def test_1871(penwidth: str):
   assert has_1 or has_1_0, \
     f"incorrect penwidth from round tripping through GML (output {gml})"
 
-@pytest.mark.skipif(shutil.which("fdp") is None, reason="fdp not available")
+@pytest.mark.skipif(which("fdp") is None, reason="fdp not available")
 def test_1876():
   """
   fdp should not rename nodes with internal names
@@ -868,7 +869,7 @@ def test_1876():
   # we should not see any internal names like "%3"
   assert "%" not in output, "internal name in fdp output"
 
-@pytest.mark.skipif(shutil.which("fdp") is None, reason="fdp not available")
+@pytest.mark.skipif(which("fdp") is None, reason="fdp not available")
 def test_1877():
   """
   fdp should not fail an assertion when processing cluster edges
@@ -971,7 +972,7 @@ def test_1855():
   assert len(y) > 4, "insufficient precision in y scale"
 
 @pytest.mark.parametrize("variant", [1, 2])
-@pytest.mark.skipif(shutil.which("gml2gv") is None, reason="gml2gv not available")
+@pytest.mark.skipif(which("gml2gv") is None, reason="gml2gv not available")
 def test_1869(variant: int):
   """
   gml2gv should be able to parse the style, outlineStyle, width and
@@ -1046,7 +1047,7 @@ def test_1906():
 
   assert "area too large" in stderr, "missing/incorrect error message"
 
-@pytest.mark.skipif(shutil.which("twopi") is None, reason="twopi not available")
+@pytest.mark.skipif(which("twopi") is None, reason="twopi not available")
 def test_1907():
   """
   SVG edges should have title elements that match their names
@@ -1063,7 +1064,7 @@ def test_1907():
   assert "<title>A&#45;&gt;B</title>" in output, \
     "element title not found in SVG"
 
-@pytest.mark.skipif(shutil.which("gvpr") is None, reason="gvpr not available")
+@pytest.mark.skipif(which("gvpr") is None, reason="gvpr not available")
 def test_1909():
   """
   GVPR should not output internal names
@@ -1189,7 +1190,7 @@ def test_1931():
   assert "line 3\nline 4" in xdot
   assert "line 5\nline 6" in xdot
 
-@pytest.mark.skipif(shutil.which("edgepaint") is None,
+@pytest.mark.skipif(which("edgepaint") is None,
                     reason="edgepaint not available")
 def test_1971():
   """
@@ -1352,8 +1353,7 @@ def test_2089_2():
   # run it
   _, _ = run_c(c_src, link=["cgraph"])
 
-@pytest.mark.skipif(shutil.which("dot2gxl") is None,
-                    reason="dot2gxl not available")
+@pytest.mark.skipif(which("dot2gxl") is None, reason="dot2gxl not available")
 def test_2092():
   """
   an empty node ID should not cause a dot2gxl NULL pointer dereference
@@ -1366,8 +1366,7 @@ def test_2092():
 
   assert p.returncode == 1, "dot2gxl crashed"
 
-@pytest.mark.skipif(shutil.which("dot2gxl") is None,
-                    reason="dot2gxl not available")
+@pytest.mark.skipif(which("dot2gxl") is None, reason="dot2gxl not available")
 def test_2093():
   """
   dot2gxl should handle elements with no ID
@@ -1396,8 +1395,7 @@ def test_2095():
   # ask Graphviz to process it
   dot("pdf", input)
 
-@pytest.mark.skipif(shutil.which("gv2gml") is None,
-                    reason="gv2gml not available")
+@pytest.mark.skipif(which("gv2gml") is None, reason="gv2gml not available")
 def test_2131():
   """
   gv2gml should be able to process basic Graphviz input
@@ -1414,8 +1412,7 @@ def test_2131():
   except subprocess.CalledProcessError as e:
     raise RuntimeError("gv2gml rejected a basic graph") from e
 
-@pytest.mark.skipif(shutil.which("gvpr") is None,
-                    reason="gvpr not available")
+@pytest.mark.skipif(which("gvpr") is None, reason="gvpr not available")
 @pytest.mark.parametrize("examine", ("indices", "tokens"))
 def test_2138(examine: str):
   """
@@ -1497,7 +1494,7 @@ def test_2179_1():
   assert "Warning: no hard-coded metrics for" not in stderr, \
     "incorrect warning triggered"
 
-@pytest.mark.skipif(shutil.which("nop") is None, reason="nop not available")
+@pytest.mark.skipif(which("nop") is None, reason="nop not available")
 def test_2184_1():
   """
   nop should not reposition labelled graph nodes
@@ -1661,7 +1658,7 @@ def test_2193():
   new = dot("canon", source=canonical)
   assert canonical == new, "canonical translation is not stable"
 
-@pytest.mark.skipif(shutil.which("gvpr") is None, reason="GVPR not available")
+@pytest.mark.skipif(which("gvpr") is None, reason="GVPR not available")
 def test_2211():
   """
   GVPR’s `index` function should return correct results
@@ -1828,7 +1825,7 @@ def test_vcxproj_inclusive(vcxproj: Path):
       "mismatch between sources in {str(vcxproj)} and {str(filters)}"
 
 @pytest.mark.xfail() # FIXME: fails on CentOS 7/8, macOS Autotools, MSBuild
-@pytest.mark.skipif(shutil.which("gvmap") is None, reason="gvmap not available")
+@pytest.mark.skipif(which("gvmap") is None, reason="gvmap not available")
 def test_gvmap_fclose():
   """
   gvmap should not attempt to fclose(NULL). This example will trigger a crash if
@@ -1860,7 +1857,7 @@ def test_gvmap_fclose():
   # pass this through gvmap
   subprocess.run(["gvmap"], input=input.encode("utf-8"), check=True)
 
-@pytest.mark.skipif(shutil.which("gvpr") is None, reason="gvpr not available")
+@pytest.mark.skipif(which("gvpr") is None, reason="gvpr not available")
 def test_gvpr_usage():
   """
   gvpr usage information should be included when erroring on a malformed command
@@ -1987,3 +1984,62 @@ def test_2272():
 
   # run the test
   run_c(c_src, link=["cgraph", "gvc"])
+
+def test_2282():
+  """
+  using the `fdp` layout with JSON output should result in valid JSON
+  https://gitlab.com/graphviz/graphviz/-/issues/2282
+  """
+
+  # locate our associated test case in this directory
+  input = Path(__file__).parent / "2282.dot"
+  assert input.exists(), "unexpectedly missing test case"
+
+  # translate this to JSON
+  output = dot("json", input)
+
+  # confirm this is valid JSON
+  json.loads(output)
+
+@pytest.mark.skipif(which("gxl2gv") is None, reason="gxl2gv not available")
+@pytest.mark.xfail()
+def test_2300_1():
+  """
+  translating GXL with an attribute `name` should not crash
+  https://gitlab.com/graphviz/graphviz/-/issues/2300
+  """
+
+  # locate our associated test case containing a node attribute `name`
+  input = Path(__file__).parent / "2300.gxl"
+  assert input.exists(), "unexpectedly missing test case"
+
+  # ask `gxl2gv` to process this
+  subprocess.check_call(["gxl2gv", input])
+
+def test_2307():
+  """
+  'id' attribute should be propagated to 'url' links in SVG output
+  https://gitlab.com/graphviz/graphviz/-/issues/2307
+  """
+
+  # locate our associated test case in this directory
+  input = Path(__file__).parent / "2258.dot"
+  assert input.exists(), "unexpectedly missing test case"
+
+  # translate this to SVG
+  svg = dot("svg", input)
+
+  # load this as XML
+  root = ET.fromstring(svg)
+
+  # the output is expected to contain a number of polygons, any of which have
+  # `url` fills should include the ID “G2”
+  polygons = root.findall(".//{http://www.w3.org/2000/svg}polygon")
+  assert len(polygons) > 0, "no polygons in output"
+
+  for polygon in polygons:
+    m = re.match(r"url\((?P<url>.*)\)$", polygon.get("fill"))
+    if m is None:
+      continue
+    assert re.search(r"\bG2_", m.group("url")) is not None, \
+      "ID G2 was not applied to polygon fill url"
